"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlayerData = void 0;
const protocol_1 = require("@skeldjs/protocol");
const objects_1 = require("./objects");
const Heritable_1 = require("./Heritable");
const events_1 = require("./events");
/**
 * Represents the player of a client connected to the room.
 *
 * See {@link PlayerDataEvents} for events to listen to.
 */
class PlayerData extends Heritable_1.Heritable {
    constructor(room, clientid) {
        super(room, clientid);
        this.stream = [];
        this.isReady = false;
        this.inScene = false;
        this.left = false;
        this.character = undefined;
        this.on("component.spawn", () => {
            if (this.spawned) {
                this.emit(new events_1.PlayerSpawnEvent(this.room, this));
            }
        });
    }
    /**
     * The player's control component.
     */
    get control() {
        var _a;
        return (_a = this.character) === null || _a === void 0 ? void 0 : _a.getComponent(objects_1.PlayerControl);
    }
    /**
     * The player's physics component.
     */
    get physics() {
        var _a;
        return (_a = this.character) === null || _a === void 0 ? void 0 : _a.getComponent(objects_1.PlayerPhysics);
    }
    /**
     * The player's movement component.
     */
    get transform() {
        var _a;
        return (_a = this.character) === null || _a === void 0 ? void 0 : _a.getComponent(objects_1.CustomNetworkTransform);
    }
    /**
     * The player's information.
     */
    get info() {
        var _a, _b;
        if (this.playerId === undefined)
            return undefined;
        return (_b = (_a = this.room.gamedata) === null || _a === void 0 ? void 0 : _a.players) === null || _b === void 0 ? void 0 : _b.get(this.playerId);
    }
    /**
     * The player ID of the player.
     */
    get playerId() {
        var _a;
        return (_a = this.control) === null || _a === void 0 ? void 0 : _a.playerId;
    }
    /**
     * Whether or not the player has fully spawned.
     */
    get spawned() {
        return !!(this.control && this.physics && this.transform);
    }
    /**
     * Whether or not the player is the host of the room they belong in.
     */
    get ishost() {
        return this.room.hostid === this.id;
    }
    /**
     * Whether or not the player is the current client's player.
     */
    get isme() {
        var _a;
        return this.id === ((_a = this.room.me) === null || _a === void 0 ? void 0 : _a.id);
    }
    /**
     * Mark as readied up to start the game.
     */
    async ready() {
        this.isReady = true;
        await this.emit(new events_1.PlayerReadyEvent(this.room, this));
        if (this.isme && !this.ishost) {
            await this.room.broadcast([new protocol_1.ReadyMessage(this.id)]);
        }
    }
}
exports.PlayerData = PlayerData;
//# sourceMappingURL=PlayerData.js.map